/* SPDX-License-Identifier: LGPL-2.1-or-later */

#ifndef KIRMES_H
#define KIRMES_H

/* WARNING: This file is autogenerated by cbindgen. Don't modify this manually. */

#define KIRMES_MAJOR 0
#define KIRMES_MINOR 0
#define KIRMES_PATCH 1


#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

/**
 * Controls whether the home area shall be grown automatically
 */
enum KirmesAutoResizeMode {
    AutoResizeOff,
    AutoResizeGrow,
    AutoResizeShrinkAndGrow,
    _AutoResizeModeMax,
};

/**
 * This type represents success and errors working with systemd's userdb.
 */
enum KirmesStatus {
    /**
     * Normal operation
     */
    KirmesStatusSuccess = 0,
    /**
     * Incorrectly set service field
     */
    KirmesStatusBadService,
    /**
     * In case uid/gid and name are specified and two records have been found
     */
    KirmesStatusConflictingRecordFound,
    /**
     * If the service doesn't support enumeration
     */
    KirmesStatusEnumerationNotSupported,
    /**
     * An internal error occurred
     */
    KirmesStatusInternalError,
    /**
     * An invalid parameter has been passed
     */
    KirmesStatusInvalidParameter,
    /**
     * An invalid record has been found, e.g. json parsing failed.
     */
    KirmesStatusInvalidRecord,
    /**
     * If no user record matching the specified UID or name is known
     */
    KirmesStatusNoRecordFound,
    /**
     * The backing subsystem is not operational
     */
    KirmesStatusServiceNotAvailable,
};

/**
 * Disposition of the user, i.e. the context it is defined in.
 */
enum KirmesUserDisposition {
    /**
     * root and nobody
     */
    UserIntrinsic,
    /**
     * statically allocated users for system services
     */
    UserSystem,
    /**
     * dynamically allocated users for system services
     */
    UserDynamic,
    /**
     * regular (typically human users)
     */
    UserRegular,
    /**
     * UID ranges allocated for container uses
     */
    UserContainer,
    /**
     * UID range allocated for foreign OS images
     */
    UserForeign,
    /**
     * Range above 2^31
     */
    UserReserved,
    _UserDispositionMax,
};

/**
 * Indicates the storage mechanism for the user’s home directory.
 */
enum KirmesUserHomeStorage {
    UserClassic,
    UserLuks,
    UserDirectory,
    UserSubvolume,
    UserFscrypt,
    UserCifs,
    _UserStorageMax,
};

/**
 * Key/Value mapping for `HashMap<String, String>`
 */
struct KirmesStrMap {
    const char *key;
    const char *val;
};

/**
 * An array holding `struct KirmesStrMap`
 * # Examples
 *
 * ```custom,{class=language-c}
 * KirmesStrVecMap m;
 * size_t i;
 *
 * for (i = 0; i < m.size; i++) {
 *     const char *key = value.vec[i].key;
 *     const char *val = value.vec[i].val;
 *     printf("%s: %s\n", key, val);
 * }
 * ```
 */
struct KirmesStrVecMap {
    const struct KirmesStrMap *vec;
    size_t size;
};

/**
 * A structure holding a string array
 *
 * # Examples
 *
 * ```custom,{class=language-c}
 * KirmesStrVec v;
 * size_t i;
 *
 * for (i = 0; i < v.size; i++) {
 *     const char *x = value.vec[i];
 *     printf("%s\n", x);
 * }
 * ```
 */
struct KirmesStrVec {
    /**
     * The string array
     */
    const char *const *vec;
    /**
     * The number of elements in the string array
     */
    size_t size;
};

/**
 * The `EncryptedKey` for PKCS#11
 */
struct KirmesPkcs11EncryptedKey {
    /**
     * PKCS#11 security token URI,
     */
    const char *uri;
    /**
     * UNIX password hash to test the key against.
     */
    const char *hashed_password;
    /**
     * Base64-encoded encrypted key
     */
    const char *data;
};

struct KirmesPkcs11EncryptedKeyVec {
    const struct KirmesPkcs11EncryptedKey *vec;
    size_t size;
};

/**
 * Authentication support with FIDO2 devices that implement the hmac-secret extension
 */
struct KirmesFido2HmacSalt {
    const char *credential;
    const char *salt;
    const char *hashed_password;
    bool uv;
    bool up;
    bool client_pin;
};

struct KirmesFido2HmacSaltVec {
    const struct KirmesFido2HmacSalt *vec;
    size_t size;
};

/**
 * Recovery keys for the user account
 */
struct KirmesRecoveryKey {
    /**
     * The type of recovery key, must be "modhex64" right now
     */
    const char *key_type;
    /**
     * A UNIX password hash of the normalized form of modhex64
     */
    const char *hashed_password;
};

struct KirmesRecoveryKeyVec {
    const struct KirmesRecoveryKey *vec;
    size_t size;
};

/**
 * Contains information that the user and the administrator should be able to see, but other users
 * should not.
 */
struct KirmesUserPrivileged {
    /**
     * A user-selected password hint in free-form text.
     */
    const char *password_hint;
    /**
     * An array of strings, each containing a hashed UNIX password string, in the format crypt(3)
     * generates.
     */
    struct KirmesStrVec hashed_passwords;
    /**
     * An array of strings, each listing an SSH public key that is authorized to access the
     * account.
     */
    struct KirmesStrVec ssh_authorized_keys;
    /**
     * An array of `Pkcs11EncryptedKey` objects.
     */
    struct KirmesPkcs11EncryptedKeyVec pkcs11_encrypted_key;
    /**
     * An array of objects, implementing authentication support with FIDO2 devices that implement
     * the hmac-secret extension.
     */
    struct KirmesFido2HmacSaltVec fido2_hmac_salt;
    /**
     * An array of objects, each defining a recovery key.
     */
    struct KirmesRecoveryKeyVec recovery_key;
};

/**
 * Resource management-related fields
 */
struct KirmesUserPerMachine {
    /**
     * An array of strings that are formatted 128-bit IDs in hex.
     */
    struct KirmesStrVec match_machine_id;
    /**
     * An array of strings that are valid hostnames.
     */
    struct KirmesStrVec match_hostname;
    /**
     * The absolute path to a world-readable copy of the user’s blob directory.
     */
    const char *blob_directory;
    /**
     * An object, which maps valid blob directory filenames to SHA256 hashes formatted as hex
     * strings.
     */
    struct KirmesStrVecMap blob_manifest;
    /**
     * The name of an icon picked by the user.
     */
    const char *icon_name;
    /**
     * A free-form location string describing the location of the user.
     */
    const char *location;
    /**
     * The shell binary to use for terminal logins of this user.
     */
    const char *shell;
    /**
     * The `umask` to set for the user’s login sessions.
     */
    uint32_t umask;
    /**
     * An array of strings, each containing an environment variable and its value to set for the
     * user’s login session.
     */
    struct KirmesStrVecMap environment;
    /**
     * The preferred timezone of the user.
     */
    const char *time_zone;
    /**
     * The preferred language/locale for the user.
     */
    const char *preferred_language;
    /**
     * The preferred languages/locales that should be used in the event that translations for the
     * `preferredLanguage` are missing
     */
    struct KirmesStrVec additional_languages;
    /**
     * An integer value in the range -20…19
     */
    int32_t nice_level;
    /**
     * If true, the user account is locked, the user may not log in.
     */
    bool locked;
    /**
     * A timestamp when record should be considered invalid for the purpose of logging in (µs
     * since the UNIX epoch 1970).
     */
    uint64_t not_before_usec;
    /**
     * The point in time after which logins shall not be permitted anymore (µs since the
     * UNIX epoch 1970).
     */
    uint64_t not_after_usec;
    /**
     * The storage mechanism for the user’s home directory.
     */
    enum KirmesUserHomeStorage storage;
    /**
     * The intended home directory disk space in bytes to assign to the user.
     */
    uint64_t disk_size;
    /**
     * Similar to diskSize but takes a relative value, but specifies a fraction of the available
     * disk space on the selected storage medium to assign to the user.
     */
    uint64_t disk_size_relative;
    /**
     * The absolute path to the skeleton directory to populate a new home directory from.
     */
    const char *skeleton_directory;
    /**
     * The UNIX access mask in the range 0…511 for the home directory when it is first created.
     */
    uint32_t access_mask;
    /**
     * The maximum number of tasks the user may start in parallel during system runtime.
     */
    uint64_t tasks_max;
    /**
     * Upper memory limits for all processes of the user.
     */
    uint64_t memory_high;
    /**
     * Upper memory limits for all processes of the user.
     */
    uint64_t memory_max;
    /**
     * CPU scheduling weights for the user’s processes (range 1…10000 ,defaults to
     * 100).
     */
    uint32_t cpu_weight;
    /**
     * IO scheduling weights for the user’s processes (range 1…10000 ,defaults to 100).
     */
    uint32_t io_weight;
    /**
     * Control nosuid mount flag of the user’s home directories.
     */
    bool mount_no_suid;
    /**
     * Control nodev mount flag of the user’s home directories.
     */
    bool mount_no_dev;
    /**
     * Control noexec mount flag of the user’s home directories.
     */
    bool mount_no_exec;
    /**
     * The Windows File Sharing domain (CIFS) to use.
     */
    const char *cifs_domain;
    /**
     * The Windows File Sharing user name (CIFS) to associate this user record with.
     */
    const char *cifs_user_name;
    /**
     * The Windows File Share service (CIFS) to mount as home directory of the user on login.
     */
    const char *cifs_service;
    /**
     * Additional mount options to pass to mount.cifs when mounting the home directory CIFS share.
     */
    const char *cifs_extra_mount_options;
    /**
     * An absolute file system path to the file, directory or block device to use for storage
     * backing the home directory.
     */
    const char *image_path;
    /**
     * The numeric UNIX user ID (UID) to use for the user.
     */
    uint32_t uid;
    /**
     * The numeric UNIX group ID (GID) to use for the user.
     */
    uint32_t gid;
    /**
     * An array of strings, each indicating a UNIX group this user shall be a member of.
     */
    struct KirmesStrVec member_of;
    /**
     * One of ext4, xfs, btrfs (possibly others) to use as file system for the user’s home
     * directory.
     */
    const char *file_system_type;
    /**
     * A lower-case, text-formatted UUID, referencing the GPT partition UUID the home directory is
     * located in.
     */
    const char *partition_uuid;
    /**
     * A lower-case, text-formatted UUID, referencing the LUKS volume UUID the home directory is
     * located in.
     */
    const char *luks_uuid;
    /**
     * A lower-case, text-formatted UUID, referencing the file system UUID the home directory is
     * located in.
     */
    const char *file_system_uuid;
    /**
     *  If true and luks storage is used, controls whether the loopback block devices, LUKS and
     *  the file system on top shall be used in discard mode.
     */
    bool luks_discard;
    /**
     * Similar to luksDiscard, it controls whether to trim/allocate the file system/backing file
     * when deactivating the home directory.
     */
    bool luks_offline_discard;
    /**
     * The cipher to use for the LUKS storage mechanism.
     */
    const char *luks_cipher;
    const char *luks_cipher_mode;
    /**
     * The volume key length in bytes to use for the LUKS storage mechanism.
     */
    uint64_t luks_volume_key_size;
    /**
     * Selects the hash algorithm to use for the PBKDF operation.
     */
    const char *luks_pbkdf_hash_algorithm;
    /**
     * The PBKDF type to use for the LUKS storage mechanism.
     */
    const char *luks_pbkdf_type;
    /**
     * The intended number of iterations for the PBKDF operation.
     */
    uint64_t luks_pbkdf_force_iterations;
    /**
     * The intended time cost for the PBKDF operation, in µs.
     */
    uint64_t luks_pbkdf_time_cost_usec;
    /**
     * The intended memory cost for the PBKDF operation, in bytes.
     */
    uint64_t luks_pbkdf_memory_cost;
    /**
     * The intended required parallel threads for the PBKDF operation.
     */
    uint64_t luks_pbkdf_parallel_threads;
    /**
     * The sector size to use for the LUKS storage mechanism, in bytes.
     */
    uint64_t luks_sector_size;
    /**
     * Controls whether the home area shall be grown automatically.
     */
    enum KirmesAutoResizeMode auto_resize_mode;
    /**
     * Configures the free disk space rebalancing weight for the home area.
     */
    uint64_t rebalance_weight;
    /**
     * Configures the authentication rate limiting enforced on the user account.
     */
    uint64_t rate_limit_interval_usec;
    /**
     * Closely related to rateLimitIntervalUSec, that puts a limit on authentication attempts
     * within the configured time interval.
     */
    uint64_t rate_limit_interval_burst;
    /**
     * Configures whether to enforce the system’s password policy when creating the home directory
     * for the user or changing the user’s password.
     */
    bool enforce_password_policy;
    /**
     * If true the user record is marked as suitable for auto-login.
     */
    bool auto_login;
    /**
     * The user’s preferred session type (i.e. x11, wayland, or other values valid for
     * `$XDG_SESSION_TYPE`).
     */
    const char *preferred_session_type;
    /**
     * The user’s preferred session launcher desktop entry file (i.e. gnome, plasma, ...)
     */
    const char *preferred_session_launcher;
    /**
     * The time in µs the per-user service manager is kept around after the user fully logged out.
     */
    uint64_t stop_delay_usec;
    /**
     *  If true all processes of the user are automatically killed when the user logs out.
     */
    bool kill_processes;
    /**
     * Encoding how much time has to pass at least between password changes of the user.
     */
    uint64_t password_change_min_usec;
    /**
     * Encodes how much time has to pass at at most between password changes of the user.
     */
    uint64_t password_change_max_usec;
    /**
     * How much time to warn the user before their password expires, in µs.
     */
    uint64_t password_change_warn_usec;
    /**
     * Encodes how much time has to pass after the password expired that the account is
     * deactivated, in µs.
     */
    uint64_t password_change_inactive_usec;
    /**
     * If true the user has to change their password on next login.
     */
    bool password_change_now;
    /**
     * An array of strings, each with an RFC 7512 compliant PKCS#11 URI referring to security
     * token (or smart card) of some form, that shall be associated with the user and may be used
     * for authentication.
     */
    struct KirmesStrVec pkcs11_token_uri;
    /**
     * An array of strings,each with a Base64-encoded FIDO2 credential ID that shall be used for
     * authentication with FIDO2 devices that implement the hmac-secret extension.
     */
    struct KirmesStrVec fido2_hmac_credential;
    /**
     * An array of strings, each corresponding to a field name that can appear in the regular or
     * perMachine sections.
     */
    struct KirmesStrVec self_modifiable_fields;
    /**
     * Similar to selfModifiableFields, but it lists blobs that the user is allowed to edit.
     */
    struct KirmesStrVec self_modifiable_blobs;
    /**
     * Similar to selfModifiableFields, but it lists fields in the privileged section that the
     * user is allowed to edit.
     */
    struct KirmesStrVec self_modifiable_privileged;
};

/**
 * A structure holding an array of `struct KirmesUserPerMachine`
 *
 * Examples
 *
 * ```custom,{class=language-c}
 * KirmesUserPerMachineVec v;
 * size_t i;
 * const char *my_machine_id = get_machine_id();
 * const char *my_hostname = get_hostname();
 *
 * for (i = 0; i < v.size; i++) {
 *     struct KirmesUserPerMachine *per_machine = &value.vec[i];
 *     bool for_me = false;
 *     size_t j;
 *
 *     for (j = 0; j < per_machine->match_machine_id.size; j++) {
 *         const char *id = per_machine->match_machine_id.vec[j];
 *         int cmp;
 *
 *         cmp = strcasecmp(id, my_machine_id);
 *         if (cmp == 0) {
 *             for_me = true;
 *         }
 *     }
 *
 *     for (j = 0; j < per_machine->match_hostname.size; j++) {
 *         const char *host = per_machine->match_hostname.vec[j];
 *         int cmp;
 *
 *         cmp = strcasecmp(host, my_hostname);
 *         if (cmp == 0) {
 *             for_me = true;
 *         }
 *     }
 *
 *     if (!for_me) {
 *         continue;
 *     }
 *
 *     // This per_machine entry is for this host
 * }
 * ```
 *
 */
struct KirmesUserPerMachineVec {
    const struct KirmesUserPerMachine *vec;
    size_t size;
};

/**
 * The binding section overrides their equally named counterparts in the regular and perMachine
 * sections.
 */
struct KirmesUserBinding {
    /**
     * The machine ID the binding is intended for (key).
     */
    const char *for_machine_id;
    /**
     * The absolute path to a world-readable copy of the user’s blob directory.
     */
    const char *blob_directory;
    /**
     * An absolute file system path to the file, directory or block device to use for storage
     * backing the home directory.
     */
    const char *image_path;
    /**
     * An absolute file system path to the home directory.
     */
    const char *home_directory;
    /**
     * A lower-case, text-formatted UUID, referencing the GPT partition UUID the home directory is
     * located in.
     */
    const char *partition_uuid;
    /**
     * A lower-case, text-formatted UUID, referencing the LUKS volume UUID the home directory is
     * located in.
     */
    const char *luks_uuid;
    /**
     * One of ext4, xfs, btrfs (possibly others) to use as file system for the user’s home
     * directory.
     */
    const char *file_system_type;
    /**
     * A lower-case, text-formatted UUID, referencing the GPT partition UUID the home directory is
     * located in.
     */
    const char *file_system_uuid;
    /**
     * The numeric UNIX user ID (UID) to use for the user.
     */
    uint32_t uid;
    /**
     * The numeric UNIX group ID (GID) to use for the user.
     */
    uint32_t gid;
    /**
     * The storage mechanism for the user’s home directory.
     */
    enum KirmesUserHomeStorage storage;
    /**
     * The cipher to use for the LUKS storage mechanism.
     */
    const char *luks_cipher;
    const char *luks_cipher_mode;
    /**
     * The volume key length in bytes to use for the LUKS storage mechanism.
     */
    uint64_t luks_volume_key_size;
};

/**
 * Structure for an array of `struct KirmesUserBinding`.
 *
 * # Examples
 *
 * ```custom,{class=language-c}
 * KirmesUserBindingVec v;
 * size_t i;
 * const char *my_machine_id = get_machine_id();
 *
 * for (i = 0; i < v.size; i++) {
 *     struct KirmesUserBinding *binding = &value.vec[i];
 *     int cmp;
 *
 *     // Check if this entry is for this machine
 *     cmp = strcasecmp(binding->for_machine_id, my_machine_id);
 *     if (cmp != 0) {
 *         continue;
 *     }
 *
 *     // This binding entry is for this machine
 * }
 * ```
 */
struct KirmesUserBindingVec {
    const struct KirmesUserBinding *vec;
    size_t size;
};

/**
 * Contains information about current runtime resource usage.
 */
struct KirmesUserStatus {
    /**
     * The machine ID the binding is intended for (key).
     */
    const char *for_machine_id;
    /**
     * The currently used disk space of the home directory in bytes.
     */
    uint64_t disk_usage;
    /**
     * The number of 'free' bytes in the disk space allotment.
     */
    uint64_t disk_free;
    /**
     * The intended home directory disk space in bytes to assign to the user.
     */
    uint64_t disk_size;
    /**
     * The upper bounds when changing the `diskSize` value, in bytes.
     */
    uint64_t disk_ceiling;
    /**
     * The lower bounds when changing the `diskSize` value, in bytes.
     */
    uint64_t disk_floor;
    /**
     * The current state of the home directory.
     */
    const char *state;
    /**
     * If true indicates that the user record is signed by a public key for which the private key
     * is available locally.
     */
    bool signed_locally;
    /**
     * This counter is increased by one on every successful authentication attempt.
     */
    uint64_t good_authentication_counter;
    /**
     * This counter is increased by one on every unsuccessfully authentication attempt.
     */
    uint64_t bad_authentication_counter;
    /**
     * The time of the last successful authentication attempt in µs since the UNIX epoch (1970).
     */
    uint64_t last_good_authentication_usec;
    /**
     * Similar, but the timestamp of the last unsuccessfully authentication attempt.
     */
    uint64_t last_bad_authentication_usec;
    /**
     * The µs timestamp since the UNIX epoch (1970) where the most recent rate limiting interval
     * has been started.
     */
    uint64_t rate_limit_begin_usec;
    /**
     * Counting the authentication attempts in the current rate limiting interval.
     */
    uint64_t rate_limit_count;
    /**
     * If true the manager of this user record determined the home directory being on removable
     * media.
     */
    bool removeable;
    /**
     * The service that defines or manages this user record.
     */
    const char *service;
};

/**
 * Structure tho hold an array of `struct KirmesUserStatus`.
 *
 * # Examples
 *
 * ```custom,{class=language-c}
 * KirmesUserStatusVec v;
 * size_t i;
 * const char *my_machine_id = get_machine_id();
 *
 * for (i = 0; i < v.size; i++) {
 *     struct KirmesUserStatus *status = &value.vec[i];
 *     int cmp;
 *
 *     // Check if this entry is for this machine
 *     cmp = strcasecmp(status->for_machine_id, my_machine_id);
 *     if (cmp != 0) {
 *         continue;
 *     }
 *
 *     // This status entry is for this machine
 * }
 * ```
 */
struct KirmesUserStatusVec {
    const struct KirmesUserStatus *vec;
    size_t size;
};

/**
 * Contains one or more cryptographic signatures of a reduced version of the user record.
 */
struct KirmesUserSignature {
    /**
     * The signature, encoded in Base64
     */
    const char *data;
    /**
     * A copy of the public key whose private key was used to make the signature, in PEM format.
     */
    const char *key;
};

/**
 * Structure for an array of `struct KirmesUserSignature`
 *
 * Examples
 *
 * ```custom,{class=language-c}
 * KirmesUserSignatureVec v;
 * size_t i;
 *
 * for (i = 0; i < v.size; i++) {
 *     struct KirmesUserSignature *signature = &value.vec[i];
 *
 *     // Access signature data and key
 * }
 *
 */
struct KirmesUserSignatureVec {
    const struct KirmesUserSignature *vec;
    size_t size;
};

/**
 * Secret user credentials, such as password or PIN information.
 */
struct KirmesUserSecret {
    /**
     * An array of strings, each containing a plain text password.
     */
    struct KirmesStrVec passwords;
    /**
     * An array of strings, each containing a plain text PIN, suitable for unlocking security
     * tokens that require that.
     */
    struct KirmesStrVec token_pin;
    /**
     * If set to true allows the receiver to use the PKCS#11 “protected authentication path” for
     * authenticating the user.
     */
    bool pkcs11_protected_authentication_path_permitted;
    /**
     * If set to true allows the receiver to use the FIDO2 “user presence” flag.
     */
    bool fido2_user_precense_permitted;
    /**
     * If set to true allows the receiver to use the FIDO2 “user verification” flag.
     */
    bool fido2_user_verification_permitted;
};

/**
 * User Records
 *
 * For a detailed explanation please visit <https://systemd.io/USER_RECORDS>
 */
struct KirmesUserRecord {
    /**
     * The UNIX user name for this record.
     */
    const char *user_name;
    /**
     * The 'realm' a user is defined in.
     */
    const char *realm;
    /**
     * The absolute path to a world-readable copy of the user’s blob directory.
     */
    const char *blob_directory;
    /**
     * An object, which maps valid blob directory filenames to SHA256 hashes formatted as hex
     * strings.
     */
    struct KirmesStrVecMap blob_manifest;
    /**
     * The real name of the user.
     */
    const char *real_name;
    /**
     * The email address of the user.
     */
    const char *email_address;
    /**
     * The name of an icon picked by the user.
     */
    const char *icon_name;
    /**
     * A free-form location string describing the location of the user.
     */
    const char *location;
    /**
     * Clarifies the disposition of the user, i.e. the context it is defined in.
     */
    enum KirmesUserDisposition disposition;
    /**
     * A timestamp indicating when the user record was last changed (µs since the epoch 1970).
     */
    uint64_t last_change_usec;
    /**
     * A timestamp indicating the point in time when password of the user was last changed (µs
     * since the epoch 1970).
     */
    uint64_t last_password_change_usec;
    /**
     * The shell binary to use for terminal logins of this user.
     */
    const char *shell;
    /**
     * The `umask` to set for the user’s login sessions.
     */
    uint32_t umask;
    /**
     * An array of strings, each containing an environment variable and its value to set for the
     * user’s login session.
     */
    struct KirmesStrVecMap environment;
    /**
     * The preferred timezone of the user.
     */
    const char *time_zone;
    /**
     * The preferred language/locale for the user.
     */
    const char *preferred_language;
    /**
     * The preferred languages/locales that should be used in the event that translations for the
     * `preferredLanguage` are missing
     */
    struct KirmesStrVec additional_languages;
    /**
     * An integer value in the range -20…19
     */
    int32_t nice_level;
    /**
     * If true, the user account is locked, the user may not log in.
     */
    bool locked;
    /**
     * A timestamp when record should be considered invalid for the purpose of logging in (µs
     * since the UNIX epoch 1970).
     */
    uint64_t not_before_usec;
    /**
     * The point in time after which logins shall not be permitted anymore (µs since the
     * UNIX epoch 1970).
     */
    uint64_t not_after_usec;
    /**
     * The storage mechanism for the user’s home directory.
     */
    enum KirmesUserHomeStorage storage;
    /**
     * The intended home directory disk space in bytes to assign to the user.
     */
    uint64_t disk_size;
    /**
     * Similar to diskSize but takes a relative value, but specifies a fraction of the available
     * disk space on the selected storage medium to assign to the user.
     */
    uint64_t disk_size_relative;
    /**
     * The absolute path to the skeleton directory to populate a new home directory from.
     */
    const char *skeleton_directory;
    /**
     * The UNIX access mask in the range 0…511 for the home directory when it is first created.
     */
    uint32_t access_mask;
    /**
     * The maximum number of tasks the user may start in parallel during system runtime.
     */
    uint64_t tasks_max;
    /**
     * Upper memory limits for all processes of the user.
     */
    uint64_t memory_high;
    /**
     * Upper memory limits for all processes of the user.
     */
    uint64_t memory_max;
    /**
     * CPU scheduling weights for the user’s processes (range 1…10000 ,defaults to
     * 100).
     */
    uint32_t cpu_weight;
    /**
     * IO scheduling weights for the user’s processes (range 1…10000 ,defaults to 100).
     */
    uint32_t io_weight;
    /**
     * Control nosuid mount flag of the user’s home directories.
     */
    bool mount_no_suid;
    /**
     * Control nodev mount flag of the user’s home directories.
     */
    bool mount_no_dev;
    /**
     * Control noexec mount flag of the user’s home directories.
     */
    bool mount_no_exec;
    /**
     * The Windows File Sharing domain (CIFS) to use.
     */
    const char *cifs_domain;
    /**
     * The Windows File Sharing user name (CIFS) to associate this user record with.
     */
    const char *cifs_user_name;
    /**
     * The Windows File Share service (CIFS) to mount as home directory of the user on login.
     */
    const char *cifs_service;
    /**
     * Additional mount options to pass to mount.cifs when mounting the home directory CIFS share.
     */
    const char *cifs_extra_mount_options;
    /**
     * An absolute file system path to the file, directory or block device to use for storage
     * backing the home directory.
     */
    const char *image_path;
    /**
     * An absolute file system path to the home directory.
     */
    const char *home_directory;
    /**
     * The numeric UNIX user ID (UID) to use for the user.
     */
    uint32_t uid;
    /**
     * The numeric UNIX group ID (GID) to use for the user.
     */
    uint32_t gid;
    /**
     * An array of strings, each indicating a UNIX group this user shall be a member of.
     */
    struct KirmesStrVec member_of;
    /**
     * One of ext4, xfs, btrfs (possibly others) to use as file system for the user’s home
     * directory.
     */
    const char *file_system_type;
    /**
     * A lower-case, text-formatted UUID, referencing the GPT partition UUID the home directory is
     * located in.
     */
    const char *partition_uuid;
    /**
     * A lower-case, text-formatted UUID, referencing the LUKS volume UUID the home directory is
     * located in.
     */
    const char *luks_uuid;
    /**
     * A lower-case, text-formatted UUID, referencing the file system UUID the home directory is
     * located in.
     */
    const char *file_system_uuid;
    /**
     *  If true and luks storage is used, controls whether the loopback block devices, LUKS and
     *  the file system on top shall be used in discard mode.
     */
    bool luks_discard;
    /**
     * Similar to luksDiscard, it controls whether to trim/allocate the file system/backing file
     * when deactivating the home directory.
     */
    bool luks_offline_discard;
    /**
     * Additional mount options to append to the default mount options for the file system in the
     * LUKS volume.
     */
    const char *luks_extra_mount_options;
    /**
     * The cipher to use for the LUKS storage mechanism.
     */
    const char *luks_cipher;
    const char *luks_cipher_mode;
    /**
     * The volume key length in bytes to use for the LUKS storage mechanism.
     */
    uint64_t luks_volume_key_size;
    /**
     * Selects the hash algorithm to use for the PBKDF operation.
     */
    const char *luks_pbkdf_hash_algorithm;
    /**
     * The PBKDF type to use for the LUKS storage mechanism.
     */
    const char *luks_pbkdf_type;
    /**
     * The intended number of iterations for the PBKDF operation.
     */
    uint64_t luks_pbkdf_force_iterations;
    /**
     * The intended time cost for the PBKDF operation, in µs.
     */
    uint64_t luks_pbkdf_time_cost_usec;
    /**
     * The intended memory cost for the PBKDF operation, in bytes.
     */
    uint64_t luks_pbkdf_memory_cost;
    /**
     * The intended required parallel threads for the PBKDF operation.
     */
    uint64_t luks_pbkdf_parallel_threads;
    /**
     * The sector size to use for the LUKS storage mechanism, in bytes.
     */
    uint64_t luks_sector_size;
    /**
     * Controls whether the home area shall be grown automatically.
     */
    enum KirmesAutoResizeMode auto_resize_mode;
    /**
     * Configures the free disk space rebalancing weight for the home area.
     */
    uint64_t rebalance_weight;
    /**
     * The service that defines or manages this user record.
     */
    const char *service;
    /**
     * Configures the authentication rate limiting enforced on the user account.
     */
    uint64_t rate_limit_interval_usec;
    /**
     * Closely related to rateLimitIntervalUSec, that puts a limit on authentication attempts
     * within the configured time interval.
     */
    uint64_t rate_limit_interval_burst;
    /**
     * Configures whether to enforce the system’s password policy when creating the home directory
     * for the user or changing the user’s password.
     */
    bool enforce_password_policy;
    /**
     * If true the user record is marked as suitable for auto-login.
     */
    bool auto_login;
    /**
     * The user’s preferred session type (i.e. x11, wayland, or other values valid for
     * `$XDG_SESSION_TYPE`).
     */
    const char *preferred_session_type;
    /**
     * The user’s preferred session launcher desktop entry file (i.e. gnome, plasma, ...)
     */
    const char *preferred_session_launcher;
    /**
     * The time in µs the per-user service manager is kept around after the user fully logged out.
     */
    uint64_t stop_delay_usec;
    /**
     *  If true all processes of the user are automatically killed when the user logs out.
     */
    bool kill_processes;
    /**
     * Encoding how much time has to pass at least between password changes of the user.
     */
    uint64_t password_change_min_usec;
    /**
     * Encodes how much time has to pass at at most between password changes of the user.
     */
    uint64_t password_change_max_usec;
    /**
     * How much time to warn the user before their password expires, in µs.
     */
    uint64_t password_change_warn_usec;
    /**
     * Encodes how much time has to pass after the password expired that the account is
     * deactivated, in µs.
     */
    uint64_t password_change_inactive_usec;
    /**
     * If true the user has to change their password on next login.
     */
    bool password_change_now;
    /**
     * An array of strings, each with an RFC 7512 compliant PKCS#11 URI referring to security
     * token (or smart card) of some form, that shall be associated with the user and may be used
     * for authentication.
     */
    struct KirmesStrVec pkcs11_token_uri;
    /**
     * An array of strings, each with a Base64-encoded FIDO2 credential ID that shall be used for
     * authentication with FIDO2 devices that implement the hmac-secret extension.
     */
    struct KirmesStrVec fido2_hmac_credential;
    /**
     * An array of strings, each indicating the type of one recovery key.
     */
    struct KirmesStrVec recovery_key_type;
    /**
     * An array of strings, each corresponding to a field name that can appear in the regular or
     * perMachine sections.
     */
    struct KirmesStrVec self_modifiable_fields;
    /**
     * Similar to selfModifiableFields, but it lists blobs that the user is allowed to edit.
     */
    struct KirmesStrVec self_modifiable_blobs;
    /**
     * Similar to selfModifiableFields, but it lists fields in the privileged section that the
     * user is allowed to edit.
     */
    struct KirmesStrVec self_modifiable_privileged;
    const struct KirmesUserPrivileged *privileged;
    struct KirmesUserPerMachineVec per_machine;
    struct KirmesUserBindingVec binding;
    struct KirmesUserStatusVec status;
    struct KirmesUserSignatureVec signature;
    const struct KirmesUserSecret *secret;
};

/**
 * The method looks up or enumerates a user record.
 *
 * * `uid`: Specifies the numeric UNIX UID to search for.
 * * `user_name`: Specifies the name of the user to search for.
 * * `service`: Mandatory argument that should be set to the service name being talked to.
 *              If you don't want to talk to a specific service, use: `io.systemd.Multiplexer`
 * * `out`: An array of `struct UserRecord`
 *
 * # Returns
 * On Success: ``KirmesStatusSuccess``
 * On Error: ``KirmesStatusBadService``
 *           ``KirmesStatusConflictingRecordFound``
 *           ``KirmesStatusEnumerationNotSupported``
 *           ``KirmesStatusInternalError``
 *           ``KirmesStatusInvalidParameter``
 *           ``KirmesStatusInvalidRecord``
 *           ``KirmesStatusNoRecordFound``
 *           ``KirmesStatusServiceNotAvailable``
 *
 */
enum KirmesStatus kirmes_get_user_record(const unsigned int *uid,
                                         const char *user_name,
                                         const char *service,
                                         struct KirmesUserRecord **out_records,
                                         size_t *num_records);

/**
 * Free a `KirmesUserRecord` array.
 *
 * * `records`: The pointer to free.
 */
void kirmes_user_record_free(struct KirmesUserRecord *records);

#endif  /* KIRMES_H */
